#include "tm1637.h"

#define _tm1637ClkLow()     gpio_clear(PORT_I2C, CLKPIN); // reset
#define _tm1637ClkHigh()    gpio_set(PORT_I2C, CLKPIN);   // set
#define _tm1637DioLow()     gpio_clear(PORT_I2C, DIOPIN); // reset
#define _tm1637DioHigh()    gpio_set(PORT_I2C, DIOPIN);   // set

const char segmentMap[] = {
    0x3f, 0x06, 0x5b, 0x4f, 0x66, 0x6d, 0x7d, 0x07, // 0-7
    0x7f, 0x6f, 0x77, 0x7c, 0x39, 0x5e, 0x79, 0x71, // 8-9, A-F
    0x00
};

void _tm1637DelayUsec(unsigned int i) {
    for (; i>0; i--) {
        for (int j = 0; j < FDEL; ++j) {
            __asm__ __volatile__("nop\n\t":::"memory");
        }
    }
}

void _tm1637Start(void) {
    _tm1637ClkHigh();
    _tm1637DioHigh();
    _tm1637DelayUsec(2);
    _tm1637DioLow();
}

void _tm1637Stop(void) {
    _tm1637ClkLow();
    _tm1637DelayUsec(2);
    _tm1637DioLow();
    _tm1637DelayUsec(2);
    _tm1637ClkHigh();
    _tm1637DelayUsec(2);
    _tm1637DioHigh();
}

void _tm1637ReadResult(void) {
    _tm1637ClkLow();
    _tm1637DelayUsec(5);
    // while (dio); // We're cheating here and not actually reading back the response.
    _tm1637ClkHigh();
    _tm1637DelayUsec(2);
    _tm1637ClkLow();
}

void _tm1637WriteByte(unsigned char b) {
    for (int i = 0; i < 8; ++i) {
        _tm1637ClkLow();
        if (b & 0x01) {
            _tm1637DioHigh();
        }
        else {
            _tm1637DioLow();
        }
        _tm1637DelayUsec(3);
        b >>= 1;
        _tm1637ClkHigh();
        _tm1637DelayUsec(3);
    }
}

// Valid brightness values: 0 - 8.
// 0 = display off.
void tm1637SetBrightness(char brightness) {
    // Brightness command:
    // 1000 0XXX = display off
    // 1000 1BBB = display on, brightness 0-7
    // X = don't care
    // B = brightness
    _tm1637Start();
    _tm1637WriteByte(0x87 + brightness);
    _tm1637ReadResult();
    _tm1637Stop();
}

void tm1637Init(void) {
    rcc_periph_clock_enable(RCCPORT);
    gpio_mode_setup(PORT_I2C, GPIO_MODE_OUTPUT, GPIO_PUPD_NONE, CLKPIN);
    gpio_mode_setup(PORT_I2C, GPIO_MODE_OUTPUT, GPIO_PUPD_NONE, DIOPIN);
    gpio_set_output_options(PORT_I2C, GPIO_OTYPE_OD, GPIO_OSPEED_HIGH, CLKPIN);
    gpio_set_output_options(PORT_I2C, GPIO_OTYPE_OD, GPIO_OSPEED_HIGH, DIOPIN);
    tm1637SetBrightness(5);
}

void tm1637DisplayDecimal(int v, int displaySeparator) {
    unsigned char digitArr[4];
    for (int i = 0; i < 4; ++i) {
        digitArr[i] = segmentMap[v % 10];
        if (i == 2 && displaySeparator) {
            digitArr[i] |= 1 << 7;
        }
        v /= 10;
    }
    _tm1637Start();
    _tm1637WriteByte(0x40);
    _tm1637ReadResult();
    _tm1637Stop();
    _tm1637Start();
    _tm1637WriteByte(0xc0);
    _tm1637ReadResult();
    for (int i = 0; i < 4; ++i) {
        _tm1637WriteByte(digitArr[3 - i]);
        _tm1637ReadResult();
    }
    _tm1637Stop();
}
// The END
